#!/bin/sh
#
# Licensed Materials - Property of IBM
# 5724-C08
# (c) Copyright International Business Machines Corp. 2012
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

#############################################################################
#
# The following function is used to execute a command, parsing and validating
# any command line options.
#
#############################################################################

runCommand()
{
    # Check the command line usage.
    if [ $# != 2 ] ; then
	echo "usage: runCommand [program] [options string]"
	return 1
    fi

    spaceDelim="___"
    optionString=""

    USE_RESPONSE_FILE="FALSE"
    if [ -n "${RESPONSE_FILE_STANZA}" -a -n "${RESPONSE_FILE_OPTION}" ]
    then
	USE_RESPONSE_FILE="TRUE"

	# Create the command response file
	RESPONSE_FILE="/opt/PolicyDirector/etc/${RESPONSE_FILE_STANZA}.rsp"
        rm -f ${RESPONSE_FILE} ${RESPONSE_FILE}.obf
	touch ${RESPONSE_FILE}
	chmod 0600 ${RESPONSE_FILE}

	OBFUSCATE_OPTION="-obfuscate"
	if [ "${RESPONSE_FILE_OBFUSCATE}" = "no" ]
	then
	    OBFUSCATE_OPTION=""
	fi
    fi

    # Break the option string into the individual options.  We also need to mask
    # out any embedded spaces so that we don't lose these.
    for maskedOption in `echo $2 | sed "s| |$spaceDelim|g" | sed "s|;| |g"`; do

	# Add any spaces back into the option
	option=`echo $maskedOption | sed "s|$spaceDelim| |g"`

	# Get the various elements from the options line.
	flag=`echo $option | cut -f 1 -d ':'`

	if [ "$flag" != "$option" ] ; then
	    valueStr=`echo $option | cut -f 2 -d ':'`
	    optional=`echo $option | cut -f 3 -d ':'`
            eval value=\"$valueStr\"

	    # If the value is not optional we need to check to ensure that
	    # the value has been supplied.

	    if [ "$optional" != "OPTIONAL" ] ; then
		if [ -z "$value" ] ; then
		    errorValue=`echo $valueStr | sed "s|[\\\\$}{]||g"`
		    echo "Error> The $errorValue variable must be defined!"
		    rm -f ${RESPONSE_FILE} ${RESPONSE_FILE}.obf
		    return 1
		fi

		if [ "${USE_RESPONSE_FILE}" = "TRUE" ]
		then
		    /opt/PolicyDirector/sbin/pdconf -f "${RESPONSE_FILE}" addentry ${OBFUSCATE_OPTION} ${RESPONSE_FILE_STANZA} $flag - <<EOF
$value
EOF
		    if [ $? -ne 0 ]
		    then
			echo "Error> pdconf failed to add an entry to the respone file ${RESPONSE_FILE}."
			rm -f ${RESPONSE_FILE} ${RESPONSE_FILE}.obf
			return 1
		    fi

		else
		    optionString="$optionString -$flag \"$value\""
		fi

	    else

		if [ ! -z "$value" ] ; then
		    if [ "${USE_RESPONSE_FILE}" = "TRUE" ]
		    then
			/opt/PolicyDirector/sbin/pdconf -f "${RESPONSE_FILE}" addentry ${OBFUSCATE_OPTION} ${RESPONSE_FILE_STANZA} $flag - <<EOF
$value
EOF
			if [ $? -ne 0 ]
			then
			    echo "Error> pdconf failed to add an entry to the respone file ${RESPONSE_FILE}."
			    rm -f ${RESPONSE_FILE} ${RESPONSE_FILE}.obf
			    return 1
			fi

		    else
			optionString="$optionString -$flag \"$value\""
		    fi
		fi
	    fi

	else

	    optionString="$optionString -$flag"

	fi

    done

    if [ "${USE_RESPONSE_FILE}" = "TRUE" ]
    then
	if [ -n "${RESPONSE_FILE_ARGS}" ]
	then
	    optionString="$optionString ${RESPONSE_FILE_ARGS}"
	fi

	optionString="$optionString -${RESPONSE_FILE_OPTION} ${RESPONSE_FILE}"
    fi

    # Execute the command.
    # echo "Executing: ${1} ${optionString}"
    eval ${1} ${optionString}
    retCode=$?

    rm -f ${RESPONSE_FILE} ${RESPONSE_FILE}.obf

    return $retCode
}

usage()
{
    echo "usage: $0 -f [options_file]"
    echo "   where:"
    echo "      options_file: file path for configuration options"
    return
}

#########################################################################
#
# MAIN
#
#########################################################################

# Check the usage.
if [ $# -ne 2 ] ; then
    usage $0
    exit 1
fi

while getopts :f: arguments
do
    case $arguments in
        f) OPTIONS_FILE=$OPTARG
           ;;
        \?) echo "$OPTARG is not a valid switch."
           usage
           exit 1
           ;;
    esac
done

# Make sure the specified options file exists.
if [ ! -f "${OPTIONS_FILE}" ] ; then
    echo "The specified options file '${OPTIONS_FILE}' is not found."
    exit 1
fi

# Load the specified options file. Change the internal field separator
# to just the newline character.
OLD_IFS=$IFS

IFS='
'

for i in `/opt/PolicyDirector/sbin/pdconf -f "${OPTIONS_FILE}" getstanza config`
do
    name=`cut -f 1 -d '=' <<EOF
$i
EOF`
    value=`cut -f 2- -d '=' <<EOF
$i
EOF`
    eval $name=\"$value\"
done

IFS=$OLD_IFS

# Make sure the configuration program is installed and exists.
if [ ! -f "${CONFIGPROGRAM}" ] ; then
    echo "The specified configuration program '${CONFIGPROGRAM}' is not found."
    exit 1
fi

eval options='$'${CONFIGOPTIONS}Options
if [ -z "${options}" ]; then
    echo "The specified configuration options '${CONFIGOPTIONS}' is not found."
    exit 1
fi

runCommand $CONFIGPROGRAM ${options}
